@@ -9,9 +9,9 @@ class PythonMixin(AptMixin):
     FabulAWS Ubuntu mixin that installs Python
     """
 
-    package_name = 'python'
+    package_name = "python"
     python_ppa = None
-    python_packages = ['python3', 'python3-dev']
+    python_packages = ["python3", "python3-dev"]
     python_install_tools = True
     python_virtualenv_version = None  # install the latest version
 
@@ -21,11 +21,11 @@ class PythonMixin(AptMixin):
         Installs the required Python tools from PyPI.
         """
 
-        sudo('curl https://bootstrap.pypa.io/get-pip.py --output /tmp/get-pip.py')
+        sudo("curl https://bootstrap.pypa.io/get-pip.py --output /tmp/get-pip.py")
         sudo("python3 /tmp/get-pip.py")
-        version = ''
+        version = ""
         if self.python_virtualenv_version:
-            version = '==%s' % self.python_virtualenv_version
+            version = "==%s" % self.python_virtualenv_version
         sudo("pip install -U virtualenv%s" % version)
 
     def setup(self):
